#include "art.h"
#include "player.h"
#include "win.h"
#include "game.h"

#include <algorithm>

std::auto_ptr<Art> Art::create(Player* player, int level, MainWindow* win) {
	/// ָΤ϶
	win->fillRect(player->type().color());

	std::auto_ptr<Art> art;
	if (player->type() == PlayerType::RED) {
		if (level == 1) art.reset(new RedArt1());
		else if (level == 2) art.reset(new RedArt2());
		else if (level == 3) art.reset(new RedArt3());
	}
	else if (player->type() == PlayerType::ORANGE) {
		if (level == 1) art.reset(new OrangeArt1());
		else if (level == 2) art.reset(new OrangeArt2());
		else if (level == 3) art.reset(new OrangeArt3());
	}
	else if (player->type() == PlayerType::YELLOW) {
		if (level == 1) art.reset(new YellowArt1());
		else if (level == 2) art.reset(new YellowArt2());
		else if (level == 3) art.reset(new YellowArt3());
	}
	else if (player->type() == PlayerType::GREEN) {
		if (level == 1) art.reset(new GreenArt1());
		else if (level == 2) art.reset(new GreenArt2());
		else if (level == 3) art.reset(new GreenArt3());
	}
	else if (player->type() == PlayerType::BLUE) {
		if (level == 1) art.reset(new BlueArt1());
		else if (level == 2) art.reset(new BlueArt2());
		else if (level == 3) art.reset(new BlueArt3());
	}
	else if (player->type() == PlayerType::PURPLE) {
		if (level == 1) art.reset(new PurpleArt1());
		else if (level == 2) art.reset(new PurpleArt2());
		else if (level == 3) art.reset(new PurpleArt3());
	}

	player->setAura();

	art->commonInit(player, win);
	art->setEndTime(0);
	art->init();
	return art;
}

void Art::event() {
	localEvent();
	if (isEnd()) {
		player_->unsetAura();
		localEnd();
	}
}

bool Art::isEnd() const {
	if (endTime_ == 0) return true;
	if (Game::instance()->ticks() > endTime_) return true;

	return false;
}

void Art::commonInit(Player* player, MainWindow* win) {
	player_ = player;
	win_ = win;
}

void Art::setEndTime(Uint32 time) {
	endTime_ = time + Game::instance()->ticks();
}

void RedArt1::init() {
  std::vector<Charactor*> bullets;
  Game::instance()->getShot(player_->color().reverse(), bullets);
  for (std::vector<Charactor*>::iterator ite = bullets.begin();
       ite != bullets.end(); ++ite)
  {
    (*ite)->die();
  }
}

void RedArt2::init() {
	player_->setEat(player_->getEat() + 40);
}

void RedArt3::init() {
	player_->setEat(player_->getEat() + 60);

	Player* opposite = player_->opposite();
	opposite->setEat(opposite->getEat() - 40);
}

void OrangeArt1::init() {
	/// @todo եåѤ褦
	setEndTime(10000);
	player_->setPilable(false);
}

void OrangeArt1::localEnd() {
	player_->setPilable(true);
}

void OrangeArt2::init() {
	/// @todo եåѤ褦
	setEndTime(10000);
	player_->setFlipping(true);
}

void OrangeArt2::localEnd() {
	player_->setFlipping(false);
}

void OrangeArt3::init() {
	/// @todo եåѤ褦
	setEndTime(1000);
	player_->setKilling(true);

        /* delete all of my shots */
        {
          std::vector<Charactor*> bullets;
          Game::instance()->getShot(player_->color(), bullets);
          for (std::vector<Charactor*>::iterator ite = bullets.begin();
               ite != bullets.end(); ++ite)
          {
            (*ite)->die();
          }
        }
}

void OrangeArt3::localEnd() {
	player_->setKilling(false);

        /* delete all enemy shots */
        {
          std::vector<Charactor*> bullets;
          Game::instance()->getShot(player_->color().reverse(), bullets);
          for (std::vector<Charactor*>::iterator ite = bullets.begin();
               ite != bullets.end(); ++ite)
          {
            (*ite)->die();
          }
        }
}


void YellowArt1::init() {
	setEndTime(500);
}

void YellowArt1::localEvent() {
	std::vector<Charactor*> bullets;
	Game::instance()->getShot(player_->color(), bullets);

	for (std::vector<Charactor*>::iterator ite = bullets.begin();
		 ite != bullets.end(); ++ite)
	{
		Point vec(player_->pnt() - (*ite)->pnt());
		double length = vec.length();
		vec.unit();
		(*ite)->setPnt((*ite)->pnt() +
					   vec * Game::instance()->spf() * 20000 / (length+10));
	}
}

void YellowArt2::init() {
	setEndTime(10000);
}

void YellowArt2::localEvent() {
	std::vector<Charactor*> bullets;
	Game::instance()->getShot(player_->color(), bullets);

	for (std::vector<Charactor*>::iterator ite = bullets.begin();
		 ite != bullets.end(); ++ite)
	{
		Point vec(player_->pnt() - (*ite)->pnt());
		double length = vec.length();
		vec.unit();
		(*ite)->setPnt((*ite)->pnt() +
					   vec * Game::instance()->spf() * 3000 / (length+10));
	}
}

void YellowArt3::init() {
	setEndTime(10000);
}

void YellowArt3::localEvent() {
	Player* opposite = player_->opposite();
	Point vec(player_->pnt() - opposite->pnt());
	vec.unit();
	opposite->setPnt(opposite->pnt() + vec * 50 * Game::instance()->spf());
}

void GreenArt1::init() {
	/// @todo եåѤ褦
	setEndTime(3000);
	player_->setDifferBulletFlipping(true);
}

void GreenArt1::localEnd() {
	player_->setDifferBulletFlipping(false);
}

void GreenArt2::init() {
	/// @todo եåѤ褦
	setEndTime(3000);
	player_->setDifferBulletAbsorping(true);
}

void GreenArt2::localEnd() {
	player_->setDifferBulletAbsorping(false);
}

void GreenArt3::init() {
	/// @todo եåѤ褦
	setEndTime(3000);
	player_->setDifferBulletFlipping(true);
	player_->setSameBulletFlipping(true);
}

void GreenArt3::localEnd() {
	player_->setDifferBulletFlipping(false);
	player_->setSameBulletFlipping(false);
}

void BlueArt1::init() {
	const Point& pnt = player_->center();
	Point size(100, 100);
	win_->source()->fillRect(Rect(pnt-size, pnt+size),
							 player_->color().color());
}

void BlueArt2::init() {
	Window tmp(win_->size(), 16, SDL_SWSURFACE);
	tmp.fillRect(player_->color().color());
	tmp.setAlpha(SDL_SRCALPHA|SDL_RLEACCEL, 128);
	win_->source()->blit(&tmp);
}

void BlueArt3::init() {
	win_->source()->fillRect(player_->color().color());
}

void PurpleArt1::init() {
  /* inaccurate, but who cares? */
  player_->increaseRank(30);
}

void PurpleArt2::init() {
	setEndTime(10000);
        /*
	firstRank_ = player_->getEat();
	player_->setEat(100);
        */
	player_->setDoping(true);
}

void PurpleArt2::localEnd() {
  /*
    player_->setEat(firstRank_);
  */
	player_->setDoping(false);
}

void PurpleArt3::init() {
	Game::instance()->addPartner(player_->color());
        /* this must be after addPartner() because
         * the new partner comes with lv5 eb
         */
        Game::instance()->setPartnerLevel(player_->color(),
                                          player_->getEblv());
	player_->setEat(0);
}
