#include "buzzeffect.h"
#include "player.h"
#include "game.h"

const Point BuzzEffect::size_(3, 3);

BuzzEffect::BuzzEffect(Player* player, const Point& pnt, CharColor col)
	: Charactor(pnt, 0, col), player_(player), firstTurn_(true)
{
	pnt_ = pnt;
	zako_ = true;
	endTicks_ = Game::instance()->ticks() + 200;
}

void BuzzEffect::move() {
	if (!alive_) return;

	if (endTicks_ < Game::instance()->ticks()) {
		die();
		return;
	}

	Point dir = player_->center() - pnt_;
	dir.unit();
	dir *= 100.0 * Game::instance()->spf();
	pnt_ += dir;

	if (pnt_.xi() < 1 || pnt_.yi() < 1 ||
		pnt_.xi() > surface_->size().xi() - 1 ||
		pnt_.yi() > surface_->size().yi() - 1)
	{
		die();
		return;
	}

	Uint32 pixel = surface_->mapRGB(col_.color());
	surface_->lock();
	surface_->putPixel(pnt_.xi(), pnt_.yi(), pixel);
	surface_->putPixel(pnt_.xi()+1, pnt_.yi(), pixel);
	surface_->putPixel(pnt_.xi(), pnt_.yi()+1, pixel);
	surface_->putPixel(pnt_.xi()-1, pnt_.yi(), pixel);
	surface_->putPixel(pnt_.xi(), pnt_.yi()-1, pixel);
	surface_->putPixel(pnt_.xi()+2, pnt_.yi(), pixel);
	surface_->putPixel(pnt_.xi(), pnt_.yi()+2, pixel);
	surface_->putPixel(pnt_.xi()-2, pnt_.yi(), pixel);
	surface_->putPixel(pnt_.xi(), pnt_.yi()-2, pixel);
	surface_->unlock();

	if (endTicks_ > Game::instance()->ticks()+150) {
		firstTurn_ = false;

		if (col_ != player_->color()) {
			Rect rec(pnt_.xi()-5, pnt_.yi()-5, pnt_.xi()+5, pnt_.yi()+5);
			surface_->fillRect(rec, col_.reverse().color());
		}
	}
}

