#include "command.h"
#include "game.h"
#include "util.h"
#include "player.h"
#include "conf.h"

#include <bulletml/bulletmlparser.h>

EnemyCommand::EnemyCommand(BulletMLParser* bulletml, Enemy* enemy)
    : BulletMLRunner(bulletml), enemy_(enemy)
{
}

EnemyCommand::EnemyCommand(BulletMLState* state, Enemy* enemy)
    : BulletMLRunner(state), enemy_(enemy)
{}

double EnemyCommand::getBulletDirection() {
	return rtod(PolarCoord(enemy_->spd()).r);
}

double EnemyCommand::getAimDirection() {
	return rtod(Game::instance()->player(enemy_->color().reverse())->center().angle(enemy_->center()));
}

double EnemyCommand::getBulletSpeed() {
	return PolarCoord(enemy_->spd()).l * 0.01;
}

double EnemyCommand::getDefaultSpeed() {
	return 1;
}

double EnemyCommand::getRank() {
	return Game::instance()->player(enemy_->color())->getRank();
}

void EnemyCommand::createSimpleBullet(double direction, double speed) {
	Game::instance()->addShot(enemy_->center(),
							  Point(PolarCoord(speed*100, dtor(direction))),
							  enemy_->color());
}

void EnemyCommand::createBullet(BulletMLState* state,
								double direction, double speed) {
	Game::instance()->addEnemy(enemy_->center(),
							   Point(PolarCoord(speed*100, dtor(direction))),
							   state, enemy_->color());
}

int EnemyCommand::getTurn() {
	return static_cast<int>(Game::instance()->getFrameTurn());
}

void EnemyCommand::doVanish() {
	enemy_->alive_ = false;
}

void EnemyCommand::doChangeDirection(double direction) {
	PolarCoord pc;
	pc.l = enemy_->spd().length();
	// ۑ邽߂̃AhzbNȉ
	if (pc.l == 0.0) pc.l = 0.00001;
	pc.r = dtor(direction);
	enemy_->spd_ = Point(pc);
}

void EnemyCommand::doChangeSpeed(double speed) {
	PolarCoord  pc(enemy_->spd());
	pc.l = speed * 100.0;
	enemy_->spd_ = Point(pc);
}

void EnemyCommand::doAccelX(double speedx) {
	enemy_->spd_.x = speedx * 100;
}

void EnemyCommand::doAccelY(double speedy) {
	enemy_->spd_.y = speedy * 100;
}

double EnemyCommand::getBulletSpeedX() {
	return enemy_->spd_.x * 0.01;
}

double EnemyCommand::getBulletSpeedY() {
	return enemy_->spd_.y * 0.01;
}

