#ifdef HAVE_CONFIG_H
#include "config.h"
#endif

#include "conf.h"
#include "util.h"

#include <sstream>
#include <iostream>
#include <fstream>

#include <cassert>

#ifdef HAVE_UNISTD_H
#include <unistd.h>
#endif

#ifdef HAVE_GETOPT_H
#include <getopt.h>
#endif

#ifdef HAVE_PWD_H
#include <pwd.h>
#endif

const std::string Conf::nameCacheFile_("name.cache");

void Conf::saveNameCache(const std::string& str) const {
    std::ofstream os(nameCacheFile_.c_str());
    os << str;
}

void Conf::loadName() {
    std::ifstream is(nameCacheFile_.c_str());
    std::string name;
    if (is && is >> name) name_ = name;
    else {
#ifdef HAVE_PWD_H
	struct passwd* pw = getpwuid(getuid());
	if (pw != NULL) name_ = std::string(pw->pw_name);
#else
	name_ = "SDM";
#endif
    }
}

void Conf::parseLongOption(const std::string& str) const {
    if (str == "--help") {
	help();
    }
    else if (str == "--version") {
	version();
    }
}

void Conf::help() const {
    std::cerr << " sdmkun [OPTION] [CONFFILE]\n"
	      << "  ʲΥץ򤹤ȥϥ˾ޤ\n"
	      << "\t-e NUM \t Ũοꤷޤ\n"
	      << "\t-w     \t ̵Ũ⡼ɤˤʤޤ\n"
	      << "\t-d NUM \t Ψǥޤ\n"
	      << "\t-l NUM \t 0100ޤǤꤷƥ٥Ǥޤ\n"
	      << "\t-c     \t ԥ塼Υץ쥤įޤ\n"
	      << "  ʲΥץ򤷤Ƥϥ˾ޤ\n"
	      << "\t-n NAME\t ̾ꤷޤ\n"
	      << "\t-s FILE\t Ͽեꤷޤ\n"
	      << "\t-v NUM \t ʾNUMɽޤ\n"
	      << "\t-f     \t ץʥ⡼ɤǵưޤ\n"
	      << "\t-p     \t ץե󥰤򤷤ޤ\n"
	      << "\t-h     \t Τإפɽޤ\n";
    exit(1);
}

void Conf::version() const {
    std::cout << "sdmkun version ?\n";
    exit(0);
}

void Conf::loadConfFile(std::istream& is) {
    static const int MAXBUF = 256;
    char line[MAXBUF];

    firstGlobal_ = 0;

    while (is.getline(line, MAXBUF)) {
	std::stringstream ss(line);
	std::string key;
	ss >> key;

	if (key == "Enemy") {
	    int id;
	    std::string actFile;

	    ss >> id >> actFile;
	    ASSERT(enemyActs_.find(id) == enemyActs_.end(),
		   "Enemy: same IDs!\n");
	    enemyActs_[id] = actFile;
	}

	else if (key == "Global") {
	    int id;
	    std::string actFile;

	    ss >> id >> actFile;
	    ASSERT(globalActs_.find(id) == globalActs_.end(),
		   "Global: same IDs!\n");
	    globalActs_[id] = actFile;
	}

	else if (key == "FirstGlobal") {
	    ss >> firstGlobal_;
	}

    }
}

Conf::Conf(int argc, char* argv[])
    : scoreFile_("dmkun.scr"),
      actionFile_("bosses.act"),
      enemys_(1),
      verbose_(0),
      watch_(false),
      name_(""),
      fastMode_(false),
      profiling_(false),
      isSlow_(false),
      slow_(1),
      level_(50),
      cpu_(false),

      default_(true),
      howToRun_(ACT)
{
    std::string extraOption = "";

#ifdef HAVE_UNISTD_H

    int optc;
    while ((optc = getopt(argc, argv, "l:a:d:e:s:hv:wn:fpc")) != -1) {

	switch (optc) {
	case ('e'):
	    std::stringstream(optarg) >> enemys_;
	    ASSERT(enemys_ > 0, "Enemys are not here.");
	    default_ = false;
	    break;

	case ('d'):
	    std::stringstream(optarg) >> slow_;
	    ASSERT(slow_ > 0, "slow level can't be under zero.");
	    default_ = false;
	    isSlow_ = true;
	    break;

	case ('l'):
	    std::stringstream(optarg) >> level_;
	    ASSERT(level_ >= 0 && level_ <= 100, "level must be 0 to 100.");
	    default_ = false;
	    break;

	case ('s'):
	    scoreFile_ = std::string(optarg);
	    break;

	case ('v'):
	    std::stringstream(optarg) >> verbose_;
	    break;

	case ('w'):
	    watch_ = true;
	    default_ = false;
	    break;

	case ('c'):
	    cpu_ = true;
	    default_ = false;
	    break;

	case ('n'):
	    name_ = std::string(optarg);
	    break;

	case ('f'):
	    fastMode_ = true;
	    break;

	case ('p'):
	    profiling_ = true;
	    break;

	case ('h'):
	{
	    help();
	}
	break;

	}
    }

    if (optind < argc) {
	extraOption = argv[optind];
    }
#else
    if (argc > 1) {
	extraOption = argv[1];
    }
#endif

    if (extraOption != "") {
	parseLongOption(extraOption);

     	std::string foot = extraOption.substr(extraOption.find_last_of("."));
      	if (foot == ".conf") {
	    std::ifstream confFile(extraOption.c_str());
	    loadConfFile(confFile);
	    default_ = false;
	    howToRun_ = CONF;
	}
	else if (foot == ".act") {	
	    actionFile_ = extraOption;
	    default_ = false;
	    howToRun_ = ACT;
	}
	else if (foot == ".xml") {
	    actionFile_ = extraOption;
	    default_ = false;
	    howToRun_ = XML;
	}
	else {
	    ASSERT(0, extraOption + ": can't understand.");
	}
    }

    if (name_ == "") loadName();
}

