#include "cpu.h"
#include "game.h"
#include "player.h"
/* rnd() */
#include "util.h"

#include <cmath>
#ifndef M_PI
# define M_PI           3.14159265358979323846  /* pi */
#endif

void CpuInput::update() const {
    Game* game = Game::instance();
    const Player* player = game->player(color_);

    if (game->turn() == lastTurn_) return;

    lastTurn_ = game->turn();

    Point mypnt = player->center();
    PosSpd posspd;
    game->getEnemyPosAndSpd(posspd, color_, mypnt);

    double spf = game->spf();

    std::vector<Point> mypnts;
    mypnts.push_back(mypnt);
    mypnts.push_back(mypnt+Point(0, -150)*spf);
    mypnts.push_back(mypnt+Point(106, -106)*spf);
    mypnts.push_back(mypnt+Point(150, 0)*spf);
    mypnts.push_back(mypnt+Point(106, 106)*spf);
    mypnts.push_back(mypnt+Point(0, 150)*spf);
    mypnts.push_back(mypnt+Point(-106, 106)*spf);
    mypnts.push_back(mypnt+Point(-150, 0)*spf);
    mypnts.push_back(mypnt+Point(-106, -106)*spf);

    double dp = -1;
    int index, minIndex =0;

    /* the veto fails randomly */
    if (vetohack <= 0)
    {
      if (rnd(10000) > 9985)
      {
        vetohack = 10 + rnd(20);
      }
    }
    else
    {
      vetohack -= 1;
    }

    for (index = 0; index < 9; index++) {
		const Point& upper = player->upperPoint();

		Point& pnt = mypnts[index];
		if (pnt.x > upper.x || pnt.x < 0 || pnt.y > upper.y || pnt.y < 0)
			continue;

		double nowDp = dangerPoint(pnt, posspd, vetohack);

		// ΤޤޤιưŪ
		//if (index == prevIndex_ && nowDp != 0.0) nowDp *= 0.9;
		if (index == 0 && nowDp != 0.0) nowDp *= 0.95;

		if (nowDp < dp || dp == -1) {
			dp = nowDp;
			minIndex = index;
		}

		// std::cerr << index << ": " << nowDp << std::endl;
    }

	if (minIndex == 0) axis_ = MSDL::Axis::NONE;
	else if (minIndex == 1) axis_ = MSDL::Axis::UP;
	else if (minIndex == 2) axis_ = MSDL::Axis::UPRIGHT;
	else if (minIndex == 3) axis_ = MSDL::Axis::RIGHT;
	else if (minIndex == 4) axis_ = MSDL::Axis::DOWNRIGHT;
	else if (minIndex == 5) axis_ = MSDL::Axis::DOWN;
	else if (minIndex == 6) axis_ = MSDL::Axis::DOWNLEFT;
	else if (minIndex == 7) axis_ = MSDL::Axis::LEFT;
	else if (minIndex == 8) axis_ = MSDL::Axis::UPLEFT;

    decision_ = false;
    prevIndex_ = minIndex;

	if (nextArt_ == 0) {
		nextArt_ = rnd(3) + 1;
	}
}

double CpuInput::dangerPoint(Point mypnt, PosSpd& posspd,
                             int vetohack) const {
    double dp = 0;
    int n = 0;

	Game* game = Game::instance();

    for (PosSpd::iterator ite = posspd.begin(); ite != posspd.end(); ite++) {
		Point pnt = ite->first;
		Point spd = ite->second;
                
		double timeLength = spd.length()/mypnt.length(pnt);
                /*
		if (timeLength <= 2) continue;
                */
		if (timeLength <= 0.25) continue;

		double angle = 
			M_PI/4.0 -
			std::fabs(std::fabs(spd.angle())-std::fabs(mypnt.angle(pnt)));

		if (angle > 0)
                {  
                  dp += timeLength*timeLength*angle*angle;
                  n += 1;
                }
    }

    /* the veto */
    if ((dp >= 100.0) || (n >= 50))
    {
      /*
      std::cerr << dp << " " << n <<"too much\n";
      */
      ;
    }
    else if (vetohack <= 0)
    {
      for (PosSpd::iterator ite = posspd.begin(); ite != posspd.end(); ite++)
      {
		Point pnt = ite->first;
		Point spd = ite->second;
                /*
                  if (mypnt.length(pnt+spd*game->spf()) < 2) return 1000000;
                */
                  if (mypnt.length(pnt+spd*game->spf()) < 4)
                  {
                    return 1000000;
                  }
		double len = mypnt.length(pnt);

                /*
                  if (len < 2) return 100000;
                */
                  if (len < 4)
                  {
                    return 100000;
                  }
      }
    }
    
    // ɤ餫ȤɺݤϷ
    Point upperLen = game->player(color_)->upperPoint() - mypnt;

/*
  dp += 1.0 / (mypnt.x * mypnt.x) + 1.0 / (mypnt.y * mypnt.y)
  + 1.0 / (upperLen.x * upperLen.x) + 1.0 / (upperLen.y * upperLen.y);
*/

      if (mypnt.x < 100) dp += 10.0 / (mypnt.x * mypnt.x);
      /*
      if (mypnt.y < 100) dp += 10.0 / (mypnt.y * mypnt.y);
      */
      if (mypnt.y < 200) dp += 40.0 / (mypnt.y * mypnt.y);
      if (upperLen.x < 100) dp += 10.0 / (upperLen.x * upperLen.x);
      if (upperLen.y < 100) dp += 10.0 / (upperLen.y * upperLen.y);

    return dp;
}

bool CpuInput::getButton(int id) const {
	if (id == 1) {
		const Player* player = Game::instance()->player(color_);
                /* don't keep the button pressing */
                if (player->getXPressed() != 0)
                  return false;
                /* this depends on the internal of Player::keyEvent()
                 * all the input checks must be before resetting
                 * the eating value
                 */
                if (player->getEat() >= 100)
                {
                  if (rnd(10) == 0)
                    return true;
                  else
                    return false;
                }
		if (nextArt_ == 1 && player->buzz() > 30) { return true; }
		else if (nextArt_ == 2 && player->buzz() > 60) { return true; }
		else if (nextArt_ == 3 && player->buzz() == 100) { return true; }
		return false;
	}
        else if (id == 0)
        {
          const Player* player = Game::instance()->player(color_);
          if (player->getEat() >= 100)
          {
            if (rnd(2) == 0)
              return true;
            else
              return false;
          }
        }
	else {
		return false;
	}
        /* should not reach here */
        return false;
}
