#include "enemy.h"
#include "conf.h"
#include "game.h"
#include <bulletml/bulletmlparser.h>
#include <bulletml/bulletmlparser-tinyxml.h>
#include "command.h"
#include "player.h"
#include "win.h"

#include <fstream>
#include <sstream>

EnemyBehavior::EnemyBehavior(const std::string& filename)
    : zako_(false),
      commandType_(Random)
{
	std::ifstream is(filename.c_str());
	ASSERT(is.is_open(), filename + "action File not Found!\n");
	parseActionFile(is);
}

void EnemyBehavior::parseActionFile(std::istream& is) {
    static const int MAXBUF = 256;
    char line[MAXBUF];

    while (!is.eof()) {
		is.getline(line, MAXBUF);
		std::istringstream iss(line);
		std::string command;
		iss >> command;

		ignoreBrankAndComment(command);
		if (command.empty()) continue;

		if (command == "Zako")
			zako_ = true;
		else if (command == "Commands") {
			std::string args;
			iss >> args;
			if (args == "List") commandType_ = List;
			else if (args == "Random") commandType_ = Random;
			else ASSERT(0, "Unknown \"Commands\" argument" + args);
		}
		else {
			BulletMLParser* bs = new BulletMLParserTinyXML(command);
			bs->build();
			commands_.push_back(bs);
		}
    }

    init();
}

void EnemyBehavior::init() {
    commandIte_ = commands_.begin();
}

BulletMLParser* EnemyBehavior::getRandomAction() {
    int num = rnd(static_cast<int>(commands_.size()));
    return commands_[num];
}

BulletMLParser* EnemyBehavior::getNextAction() {
    BulletMLParser* ech = *commandIte_;
    commandIte_++;
    if (commandIte_ == commands_.end()) commandIte_ = commands_.begin();
    return ech;
}

Enemy::Enemy (const Point& pnt, Window* graph,
			  EnemyBehavior* behavior, CharColor col)
    : Charactor(pnt, graph, col), behavior_(behavior)
{
    zako_ = behavior_->isZako();
    behavior_->init();

	waitTime_ = Game::instance()->ticks() + 1500;
}

Enemy::Enemy (const Point& pnt, const Point& spd, Window* graph,
			  BulletMLState* state, CharColor col)
    : Charactor(pnt, graph, col), behavior_(0)
{
	com_.reset(new EnemyCommand(state, this));

    spd_ = spd;
    zako_ = true;
}

Enemy::~Enemy() {
}

void Enemy::endCommand() {
	com_.reset(0);
	waitTime_ = Game::instance()->ticks() + 1000;
}

void Enemy::huntEvent() {
	if (com_.get() != 0) {
        com_->run();
        if (com_->isEnd()) {
	        com_.reset(0);
	        endCommand();
	    }
    }
	else {
		// behavior ʤŨ
		if (behavior_ == 0) {
		}
		else {
			if (waitTime_ > Game::instance()->ticks()) {
				if (waitTime_ < Game::instance()->ticks() + 1000) {
					int num = (waitTime_ - Game::instance()->ticks()) / 10;
					if (num > 1) {
						Point size(num, num);
						Window tmp(size, 16, SDL_SWSURFACE);
						tmp.fillRect(col_.color());
						tmp.setAlpha(SDL_SRCALPHA|SDL_RLEACCEL, 255-num*2);
						surface_->blit(&tmp, Point(center()-size/2));
					}
				}
			}
			else {
				nextBullet();
			}
		}
    }
}

void Enemy::nextBullet() {
	if (waitTime_ <= Game::instance()->ticks()) {
		BulletMLParser* bulletml;
		if (behavior_->isRandomAction()) {
			bulletml = behavior_->getRandomAction();
		}
		else
			bulletml = behavior_->getNextAction();

		com_.reset(new EnemyCommand(bulletml, this));
	}
	else {
	}
}


void Enemy::hitByPlayer (Player* bc) {
	if (col_ == bc->color() && zako_)
		alive_ = false;
}

void Enemy::hitByEnemy (Enemy*) {
}

void Enemy::hitByShot (Shot*) {
}

EnemyMgr::EnemyMgr() {}

void EnemyMgr::init() {
    normalEnemyBehavior_.reset(new EnemyBehavior("bosses.act"));
    hardEnemyBehavior_.reset(new EnemyBehavior("bosses.act.hard"));

    /* this is stupid, but who cares? */
    ebhack0.reset(new EnemyBehavior("act.d/act-0.act"));
    ebhack1.reset(new EnemyBehavior("act.d/act-1.act"));
    ebhack2.reset(new EnemyBehavior("act.d/act-2.act"));
    ebhack3.reset(new EnemyBehavior("act.d/act-3.act"));
    ebhack4.reset(new EnemyBehavior("act.d/act-4.act"));
    ebhack5.reset(new EnemyBehavior("act.d/act-5.act"));
    ebhack6.reset(new EnemyBehavior("act.d/act-6.act"));
    ebhack7.reset(new EnemyBehavior("act.d/act-7.act"));
    ebhack8.reset(new EnemyBehavior("act.d/act-8.act"));
    ebhack9.reset(new EnemyBehavior("act.d/act-9.act"));
    ebhack10.reset(new EnemyBehavior("act.d/act-10.act"));
}    

EnemyBehavior* EnemyMgr::getNormalBehavior() {
	return ebhack5.get();
	/*
	  return normalEnemyBehavior_.get();
	*/
}

EnemyBehavior* EnemyMgr::getHardBehavior() {
	return ebhack10.get();
	/*
	  return hardEnemyBehavior_.get();
	*/
}

EnemyBehavior* EnemyMgr::getLevelBehavior(int level)
{
	if (level <= 0)
		return ebhack0.get();
	else if (level == 1)
		return ebhack1.get();
	else if (level == 2)
		return ebhack2.get();
	else if (level == 3)
		return ebhack3.get();
	else if (level == 4)
		return ebhack4.get();
	else if (level == 5)
		return ebhack5.get();
	else if (level == 6)
		return ebhack6.get();
	else if (level == 7)
		return ebhack7.get();
	else if (level == 8)
		return ebhack8.get();
	else if (level == 9)
		return ebhack9.get();
	else
		return ebhack10.get();

	/* should not reach here */
	return ebhack0.get();
}
