#include "game.h"
#include "util.h"
#include "conf.h"
#include "cpu.h"
#include "char.h"
#include "enemy.h"
#include "player.h"
#include "command.h"
#include "shot.h"
#include "bulletml/bulletmlparser.h"
#include "info.h"
#include "win.h"
#include "select.h"
#include "buzzeffect.h"

#include <algorithm>
#include <functional>
#include <iostream>
#include <fstream>
#include <string>
#include <ctime>
#include <sstream>
#include <cassert>
#include <cstdlib>
#include <stdexcept>

#include <SDL.h>

int Game::title() {
/*
    screen_->loadBMP("bmp/title.bmp");
    screen_->flip();
*/
    screen_->clear();
    screen_->flip();

	if (select() != 0) { return -1; }
    return 0;
}

void Game::drawTrace(const Point& pnt, CharColor col) {
	screen_->source()->blit(traceBall_[col].get(), pnt);
}

int Game::select() {
	Select sel(screen_.get(),
			   dynamic_cast<MSDL::Input*>(playerWhiteInput_.get()));
	if (sel.selectMenu() != 0) return -1;
	if (sel.selectPlayerType() != 0) return -1;

	if (sel.vsCpu()) {
		playerBlackInput_.reset(new CpuInput(BLACK));
	}
	else {
		playerBlackInput_.reset(
			MSDL::generateInputFromConf("2Pkey.conf").release());
	}

	Point sizew(playerHalog_[WHITE]->size());
	playerg_[WHITE].reset(new Window(sizew, 16, SDL_SWSURFACE));
	playerg_[WHITE]->blit(playerHalog_[WHITE].get());
	playerColorCoreg_[WHITE].reset(new Window(*playerCoreg_));
	playerColorCoreg_[WHITE]->setMask(sel.playerType(WHITE).color());
	playerg_[WHITE]->blit(playerColorCoreg_[WHITE].get(),
						  (sizew - playerColorCoreg_[WHITE]->size()) / 2);

	Point sizeb(playerHalog_[BLACK]->size());
	playerg_[BLACK].reset(new Window(sizeb, 16, SDL_SWSURFACE));
	playerg_[BLACK]->blit(playerHalog_[BLACK].get());
	playerColorCoreg_[BLACK].reset(new Window(*playerCoreg_));
	playerColorCoreg_[BLACK]->setMask(sel.playerType(BLACK).color());
	playerg_[BLACK]->blit(playerColorCoreg_[BLACK].get(),
						  (sizeb - playerColorCoreg_[BLACK]->size()) / 2);

	Color trans(0, 255, 255);
	playerg_[WHITE]->setColorKey(SDL_SRCCOLORKEY, trans);
	playerg_[BLACK]->setColorKey(SDL_SRCCOLORKEY, trans);

	playerg_[WHITE]->convertDisplayFormat();
	playerg_[BLACK]->convertDisplayFormat();

	player_[WHITE].reset(new Player (Point (100, 350), playerg_[WHITE].get(),
									 *playerWhiteInput_, WHITE,
									 sel.playerType(WHITE), 2));
	player_[BLACK].reset(new Player (Point (200, 350), playerg_[BLACK].get(),
									 *playerBlackInput_, BLACK,
									 sel.playerType(BLACK), 2));

	return 0;
}

void Game::endEv() {
	SDL_Event ev;
	while (SDL_PollEvent(&ev)) {
		if (ev.type == SDL_QUIT) {
			end_ = true;
			quit_ = true;
		}
	}
}

void Game::init () {
	int ret = SDL_Init(SDL_INIT_VIDEO);
    ASSERT(ret == 0, "cannot init video\n");
    ret = SDL_Init(SDL_INIT_TIMER);
    ASSERT(ret == 0, "cannot init timer\n");
    ret = SDL_Init(SDL_INIT_JOYSTICK);
    ASSERT(ret == 0, "cannot init joystick\n");

	std::atexit(SDL_Quit);

    SDL_WM_SetCaption("siro to kuro no danmakukun", "skdmkun");

    srand(time(0));

    start_ = false;
    end_ = false;
    turn_ = 0;

    try {
		std::ifstream is("screen");
		if (is) {
			int x, y;
			is >> x >> y;
			std::string full;
			is >> full;
			Uint32 addFlag = 0;
			if (full != "") addFlag = SDL_FULLSCREEN;

			screen_.reset(new MainWindow("bmp/black.bmp", x, y, addFlag));
		}
		else {
			screen_.reset(new MainWindow("bmp/black.bmp"));
		}
		screen_->printInformation(std::cout);

		info_.reset(new Info(screen_->source()));

		playerCoreg_.reset(new Window("bmp/ball_w.bmp"));
		playerHalog_[WHITE].reset(new Window("bmp/halo_w.bmp"));
		playerHalog_[BLACK].reset(new Window("bmp/halo_b.bmp"));
		enemygBoss_[WHITE].reset(new Window("bmp/enemy_white.bmp"));
		enemygZako_[WHITE].reset(new Window("bmp/ball_wg.bmp"));
		enemygBullet_[WHITE].reset(new Window("bmp/ball_wr.bmp"));
		enemygBoss_[BLACK].reset(new Window("bmp/enemy_black.bmp"));
		enemygZako_[BLACK].reset(new Window("bmp/ball_bg.bmp"));
		enemygBullet_[BLACK].reset(new Window("bmp/ball_br.bmp"));
		shotg_[WHITE].reset(new Window("bmp/ball_w.bmp"));
		shotg_[BLACK].reset(new Window("bmp/ball_b.bmp"));
		traceBall_[WHITE].reset(new Window("bmp/ball_white.bmp"));
		traceBall_[BLACK].reset(new Window("bmp/ball_black.bmp"));

		Color trans(255, 0, 255);
		playerCoreg_->setColorKey(SDL_SRCCOLORKEY, Color(0, 255, 255));
		enemygBoss_[WHITE]->setColorKey(SDL_SRCCOLORKEY, trans);
		enemygZako_[WHITE]->setColorKey(SDL_SRCCOLORKEY, trans);
		enemygBullet_[WHITE]->setColorKey(SDL_SRCCOLORKEY, trans);
		enemygBoss_[BLACK]->setColorKey(SDL_SRCCOLORKEY, trans);
		enemygZako_[BLACK]->setColorKey(SDL_SRCCOLORKEY, trans);
		enemygBullet_[BLACK]->setColorKey(SDL_SRCCOLORKEY, trans);
		shotg_[WHITE]->setColorKey(SDL_SRCCOLORKEY, Color(0, 255, 255));
		shotg_[BLACK]->setColorKey(SDL_SRCCOLORKEY, trans);
		traceBall_[WHITE]->setColorKey(SDL_SRCCOLORKEY, trans);
		traceBall_[BLACK]->setColorKey(SDL_SRCCOLORKEY, trans);
		traceBall_[WHITE]->setAlpha(SDL_SRCALPHA|SDL_RLEACCEL, 4);
		traceBall_[BLACK]->setAlpha(SDL_SRCALPHA|SDL_RLEACCEL, 2);
		playerCoreg_->convertDisplayFormat();
		enemygBoss_[WHITE]->convertDisplayFormat();
		enemygZako_[WHITE]->convertDisplayFormat();
		enemygBullet_[WHITE]->convertDisplayFormat();		enemygBoss_[BLACK]->convertDisplayFormat();
		enemygZako_[BLACK]->convertDisplayFormat();
		enemygBullet_[BLACK]->convertDisplayFormat();
		shotg_[WHITE]->convertDisplayFormat();
		shotg_[BLACK]->convertDisplayFormat();
		traceBall_[WHITE]->convertDisplayFormat();
		traceBall_[BLACK]->convertDisplayFormat();

		playerWhiteInput_.reset(
			MSDL::generateInputFromConf("1Pkey.conf").release());

		enemyMgr_.init();
    }
	catch (std::runtime_error runerror) {
		std::cerr << runerror.what() << std::endl;
		SDL_Quit();
	}
    catch (...) {
		SDL_Quit();
    }
}

void Game::addPartner(CharColor col) {
	/* depends on the real value of col --- not a good practice */
	Enemy* enemy = new Enemy(Point(50 + rnd(200), 50 + rnd(100)),
							 enemygBoss_[col].get(),
							 enemyMgr_.getNormalBehavior(), col);
	enemy_.push_front(enemy);
}

void Game::addBuzzEffect(std::auto_ptr<BuzzEffect> be) {
	buzzEffects_.push_back(be.release());
}

void Game::enemyInit() {
    const Conf* conf = Conf::instance();
    int enemys = conf->enemys();

    if (conf->gameType() != Conf::CONF) {
		for (int i = 0; i < enemys; i++) {
			Enemy* enemy;
			enemy = new Enemy(Point(155, 100+i*10), enemygBoss_[WHITE].get(),
							  enemyMgr_.getNormalBehavior(), WHITE);
			enemy_.push_back(enemy);
			enemy = new Enemy(Point(145, 100+i*10), enemygBoss_[BLACK].get(),
							  enemyMgr_.getNormalBehavior(), BLACK);
			enemy_.push_back(enemy);
		}
    }
}

void Game::reinit() {
    end_ = false;
    turn_ = 0;

    delete_clear(enemy_);
    delete_clear(shot_);
	delete_clear(buzzEffects_);

	screen_->reset();
}

void Game::addShot(const Point& pnt, const Point& spd, CharColor col) {
	Shot* shot = new Shot(pnt, spd, shotg_[col].get(), col);

    shot_.push_back(shot);
}

void Game::addEnemy(const Point& pnt, const Point& spd,
					BulletMLState* state,
					CharColor col) {
    Enemy* enemy;
	const std::vector<BulletMLNode*>& acts = state->getNode();
	std::vector<BulletMLNode*>::const_iterator ite;
	for (ite = acts.begin(); ite != acts.end(); ite++) {
		const BulletMLNode* act = *ite;
		if (act->findNode(BulletMLNode::fire)) break;
		if (act->findNode(BulletMLNode::fireRef)) break;
		if (act->findNode(BulletMLNode::actionRef)) break;
	}

    if (ite != acts.end()) {
		enemy = new Enemy(pnt, spd, enemygZako_[col].get(),
						  state, col);
    }
    else {
		enemy =
			new Enemy(pnt, spd, enemygBullet_[col].get(),
					  state , col);
    }
    enemy_.push_back(enemy);
}

void Game::setPartnerLevel(CharColor col, int eblv) {
	std::vector<Enemy*> partners;
	getPartner(col, partners);

	EnemyBehavior* eb = enemyMgr_.getLevelBehavior(eblv);
	std::for_each(partners.begin(), partners.end(),
				  std::bind2nd(std::mem_fun(&Enemy::setBehavior), eb));
}

int Game::start() {
    startTicks_ = ticks_ = SDL_GetTicks();

	enemyInit();
    updateWindow();
    clearWindow();

    start_ = true;

    while (!end_) {
		timerEv();
		playerEv();
		enemyEv();
		shotEv();
		infoEv();
		updateWindow();
		clearWindow();
		hitEv();
		deadEv();
		endEv();
    }

	if (quit_) return -1;

	std::cout << turn_*1000/(ticks_-startTicks_) << "fps" << std::endl;

	waitInputRelease();
    start_ = false;

	if (0 != waitAnyInput()) return -1;

    return 0;
}

int Game::waitAnyInput() {
	do {
		SDL_Delay(50);
		SDL_PumpEvents();
		endEv();
		if (quit_) return -1;
		if (playerWhiteInput_->getButton(3)) return -1;
	} while (!(playerWhiteInput_->getButton(0) ||
			   playerWhiteInput_->getButton(1)));

	return 0;
}

void Game::waitInputRelease() {
	while (playerWhiteInput_->getAnyButton()) {
        SDL_Delay(50);
        SDL_PumpEvents();
    }
}

Game::Game()
    : joystickMgr_(new MSDL::JoystickMgr)
{}

Game::~Game() {
	delete_clear(enemy_);
	delete_clear(shot_);
	delete_clear(buzzEffects_);
}

void Game::timerEv() {
    Uint32 ticks, elapsed;
    do {
		ticks = SDL_GetTicks();
		elapsed = ticks - ticks_;
    } while (elapsed == 0);
	spf_ = elapsed / 1000.0;

    turn_++;
    ticks_ = ticks;

    int verbose = Conf::instance()->verbose();
    if (verbose) {
		if (turn_%verbose == 0) {
			std::cout << turn_ << "turn. "
					  << ticks_ << "ticks. "
					  << 1.0 / spf_ << " fps. "
					  << shot_.size() << "shots."
					  << enemy_.size() << "enemies."
					  << buzzEffects_.size() << "buzz effects.\n";
		}
    }
}

void Game::playerEv() {
    player_[WHITE]->event();
    player_[BLACK]->event();
}

void Game::enemyEv() {
    std::for_each(enemy_.begin(), enemy_.end(),
				  std::mem_fun(&Enemy::huntEvent));
}

void Game::shotEv() {
}

void Game::infoEv() {
	for (int i = 0; i < player_[WHITE]->lives(); i++) {
		screen_->blit(playerColorCoreg_[WHITE].get(), Point(2+i*10, 355));
	}
	for (int i = 0; i < player_[BLACK]->lives(); i++) {
		screen_->blit(playerColorCoreg_[BLACK].get(), Point(290-i*10, 355));
	}
}

void Game::deadEv() {
    std::list<Shot*>::iterator ites;
    std::list<Enemy*>::iterator itee;

    for (ites = shot_.begin(); ites != shot_.end();) {
		Shot* ch = *ites;

		if (!ch->alive()) {
			delete ch;
			ites = shot_.erase(ites);
		}
		else {
			++ites;
		}
	}

    for (itee = enemy_.begin(); itee != enemy_.end();) {
		Enemy* ch = *itee;

		if (!ch->alive()) {
			delete ch;
			itee = enemy_.erase(itee);
		}
		else {
			++itee;
		}
	}

	while (!buzzEffects_.empty() && !buzzEffects_.front()->alive()) {
		delete buzzEffects_.front();
		buzzEffects_.pop_front();
	}
}

void Game::hitEv() {
    std::list<Shot*>::iterator ites;
    for (ites = shot_.begin(); ites != shot_.end(); ites++) {
		Shot* sh = *ites;
		if (player_[WHITE]->isHit(sh)) {
			sh->hit(player_[WHITE].get());
			player_[WHITE]->hit(sh);
		}
		if (player_[BLACK]->isHit(sh)) {
			sh->hit(player_[BLACK].get());
			player_[BLACK]->hit(sh);
		}
    }
    std::list<Enemy*>::iterator itee;
    for (itee = enemy_.begin(); itee != enemy_.end(); itee++) {
		Enemy* en = *itee;
		if (player_[WHITE]->isHit(en)) {
			en->hit(player_[WHITE].get());
			player_[WHITE]->hit(en);
		}
		if (player_[BLACK]->isHit(en)) {
			en->hit(player_[BLACK].get());
			player_[BLACK]->hit(en);
		}
    }
}

void Game::clearWindow() {
	screen_->clear();
}


void Game::updateWindow() {
    std::for_each(enemy_.begin(), enemy_.end(),
				  std::mem_fun(&Enemy::move));
    std::for_each(enemy_.begin(), enemy_.end(),
				  std::mem_fun(&Enemy::wallEvent));
    std::for_each(shot_.begin(), shot_.end(),
				  std::mem_fun(&Shot::move));
    std::for_each(shot_.begin(), shot_.end(),
				  std::mem_fun(&Shot::wallEvent));
	std::for_each(buzzEffects_.begin(), buzzEffects_.end(),
				  std::mem_fun(&BuzzEffect::move));
	player_[WHITE]->move();
	player_[BLACK]->move();
	player_[WHITE]->wallEvent();
	player_[BLACK]->wallEvent();

    /* // ꡼󥷥åȤȤ
	   if (turn_%20 == 0) {
	   std::ostringstream oss;
	   oss << "dump/" << turn_ << ".bmp";
	   screen_->save(oss.str());
	   }
    */

    screen_->flip();
}

void Game::getEnemyPosAndSpd(std::vector<std::pair<Point, Point> >& out,
							 CharColor color, Point pnt) const
{
    std::list<Enemy*>::const_iterator itee;
    for (itee = enemy_.begin(); itee != enemy_.end(); itee++) {
		const Enemy* ene = *itee;
		if (color != ene->color() && pnt.length(ene->pnt()) < 100)
			out.push_back(std::make_pair(ene->center(), ene->spd()));
    }
    std::list<Shot*>::const_iterator ites;
    for (ites = shot_.begin(); ites != shot_.end(); ites++) {
		const Shot* shot = *ites;
		if (color != shot->color() && pnt.length(shot->pnt()) < 100)
			out.push_back(std::make_pair(shot->center(), shot->spd()));
    }
}

void Game::getShot(CharColor col, std::vector<Charactor*>& shots) {
	for (std::list<Shot*>::iterator ites = shot_.begin();
		 ites != shot_.end(); ++ites)
	{
		if (col == (*ites)->color()) {
			shots.push_back(*ites);
		}
	}

	for (std::list<Enemy*>::iterator itee = enemy_.begin();
		 itee != enemy_.end(); ++itee)
	{
		if ((*itee)->isZako() && col == (*itee)->color()) {
			shots.push_back(*itee);
		}
	}
}

void Game::getPartner(CharColor col, std::vector<Enemy*>& enemies) {
	for (std::list<Enemy*>::iterator ite = enemy_.begin();
		 ite != enemy_.end(); ++ite)
	{
		if (col == (*ite)->color() && !(*ite)->isZako()) {
			enemies.push_back(*ite);
		}
		// Partner  push_front ƤϤ
		/* Wrong.  If the list is
		 *
		 * 1: white partner
		 * 2: black partner
		 *
		 * then this gives nothing to the black player.
		 */
		/*
		  else {
		  break;
		  }
		*/
	}
}

