#ifndef BOHA_GAME
#define BOHA_GAME

#include "config.h"
#include "singleton.h"
#include "enemy.h"
#include "charcolor.h"
#include "mysdl_padinput.h"
#include "mysdl_joystick.h"

#include <list>
#include <vector>
#include <deque>
#include <sstream>

#include <SDL.h>

#include <boost/smart_ptr.hpp>

class MainWindow;
class Window;
class Charactor;
class Player;
class Enemy;
class Shot;
class EnemyCommand;
class Info;
class BuzzEffect;

// 󥰥ȥ
class Game : public Singleton<Game> {
public:
    Game();


    // 餬֤ä齪λ
    int title();
	int select();
    void init();
    void reinit();
    void enemyInit();
    int start();

    double fps() const { return 1.0 / spf_; }
	double spf() const { return spf_; }
    int turn() const { return turn_; }
    Uint32 ticks() const { return ticks_; }
    int shotNum() const { return shot_.size(); }
	int getFrameTurn() const {
		return ticks_ / 16;
	}

    ~Game();

    Player* player(CharColor col) const { return player_[col].get(); }

    void addShot(const Point& pnt, const Point& spd, CharColor col);
    void addEnemy(const Point& pnt, const Point& spd, BulletMLState* state,
				  CharColor col);
	void addPartner(CharColor col);
	void addBuzzEffect(std::auto_ptr<BuzzEffect> be);
	void setPartnerLevel(CharColor col, int eblv);

    bool isStart() const { return start_; }
    bool isEnd() const { return end_; }
    void end() { end_ = true; }

	/// cpu ѥ᥽å
    void getEnemyPosAndSpd(std::vector<std::pair<Point, Point> >& out,
						   CharColor color, Point pnt) const;

	void drawTrace(const Point& pnt, CharColor col);

	/**
	 * @param shots ϰ
	 */
	void getShot(CharColor col, std::vector<Charactor*>& shots);
	/**
	 * @param enemies ϰ
	 */
	void getPartner(CharColor col, std::vector<Enemy*>& enemies);

	MainWindow* screen() { return screen_.get(); }

	Window* playerGraphic() { return playerg_[WHITE].get(); }

public:
	void endEv();

public:
	int waitAnyInput();
	void waitInputRelease();

private:
    void timerEv();
    void playerEv();
    void enemyEv();
    void shotEv();
    void hitEv();
	void infoEv();
    void updateWindow();
    void clearWindow();
    void deadEv();

private:
    std::auto_ptr<MainWindow> screen_;

    std::auto_ptr<Player> player_[2];
    std::list<Enemy*> enemy_;
    std::list<Shot*> shot_;
	std::deque<BuzzEffect*> buzzEffects_;

	std::auto_ptr<Window> playerg_[2];
	std::auto_ptr<Window> playerColorCoreg_[2];
	std::auto_ptr<Window> playerCoreg_, playerHalog_[2];
    std::auto_ptr<Window> enemygBoss_[2], enemygZako_[2], enemygBullet_[2];
    std::auto_ptr<Window> shotg_[2];

	std::auto_ptr<Window> traceBall_[2];

	double spf_;

    int turn_;
    Uint32 ticks_;
    Uint32 startTicks_;

    bool start_;
    bool end_;
	bool quit_;

	std::auto_ptr<MSDL::InputBase> playerWhiteInput_, playerBlackInput_;

    EnemyMgr enemyMgr_;

	std::auto_ptr<Info> info_;

	std::auto_ptr<MSDL::JoystickMgr> joystickMgr_;
};

#endif // BOHA_GAME
