#include "input.h"
#include "util.h"

#include <SDL.h>

#include <algorithm>
#include <functional>

Input::Input() {}

Input::~Input() {}

Input::Axis KeyInput::getAxis() const {
    Uint8* keystate = SDL_GetKeyState(0);

    Axis axis = NONE;

    // ǽ˾岼
    if (keystate[SDLK_UP] == SDL_PRESSED) axis = UP;
    else if (keystate[SDLK_DOWN] == SDL_PRESSED) axis = DOWN;

    // Ͼ岼򸫤ʤ
    if (keystate[SDLK_RIGHT] == SDL_PRESSED) {
	if (axis == UP) axis = UPRIGHT;
	else if (axis == DOWN) axis = DOWNRIGHT;
	else axis = RIGHT;
    }
    else if (keystate[SDLK_LEFT] == SDL_PRESSED) {
	if (axis == UP) axis = UPLEFT;
	else if (axis == DOWN) axis = DOWNLEFT;
	else axis = LEFT;
    }

    return axis;
}

bool KeyInput::getDecision() const {
    Uint8* keystate = SDL_GetKeyState(0);

    if (keystate[SDLK_LSHIFT] == SDL_PRESSED ||
	keystate[SDLK_RETURN] == SDL_PRESSED ||
	keystate['z'] == SDL_PRESSED ||
	keystate['x'] == SDL_PRESSED ||
	keystate['c'] == SDL_PRESSED) return true;
    return false;
}

bool KeyInput::getCancel() const {
    Uint8* keystate = SDL_GetKeyState(0);

    if (keystate[SDLK_ESCAPE] == SDL_PRESSED) return true;
    return false;
}

bool KeyInput::getAButton() const {
    Uint8* keystate = SDL_GetKeyState(0);

    if (keystate['z'] == SDL_PRESSED) return true;
    return false;
}

bool KeyInput::getBButton() const {
    Uint8* keystate = SDL_GetKeyState(0);

    if (keystate['x'] == SDL_PRESSED) return true;
    return false;
}

bool KeyInput::getCButton() const {
    Uint8* keystate = SDL_GetKeyState(0);

    if (keystate['c'] == SDL_PRESSED) return true;
    return false;
}

#define JOYSTICK_PLAY_WIDTH 8192

Input::Axis JoystickInput::getAxis() const {
    Axis axis = NONE;

    // ǽ˾岼
    if(SDL_JoystickGetAxis(joystick_, 1) < -JOYSTICK_PLAY_WIDTH) axis = UP;
    else if(SDL_JoystickGetAxis(joystick_, 1) > JOYSTICK_PLAY_WIDTH) axis = DOWN;

    // Ͼ岼򸫤ʤ
    if (SDL_JoystickGetAxis(joystick_, 0) > JOYSTICK_PLAY_WIDTH) {
	if (axis == UP) axis = UPRIGHT;
	else if (axis == DOWN) axis = DOWNRIGHT;
	else axis = RIGHT;
    }
    else if (SDL_JoystickGetAxis(joystick_, 0) < -JOYSTICK_PLAY_WIDTH) {
	if (axis == UP) axis = UPLEFT;
	else if (axis == DOWN) axis = DOWNLEFT;
	else axis = LEFT;
    }

    return axis;
}

bool JoystickInput::getDecision() const {
    if(SDL_JoystickGetButton(joystick_, 0) == SDL_PRESSED ||
       SDL_JoystickGetButton(joystick_, 1) == SDL_PRESSED ||
       SDL_JoystickGetButton(joystick_, 2) == SDL_PRESSED) return true;

    return false;
}

bool JoystickInput::getCancel() const {
    return false;
}

bool JoystickInput::getAButton() const {
    if(SDL_JoystickGetButton(joystick_, 0) == SDL_PRESSED) return true;
    return false;
}

bool JoystickInput::getBButton() const {
    if(SDL_JoystickGetButton(joystick_, 1) == SDL_PRESSED) return true;
    return false;
}

bool JoystickInput::getCButton() const {
    if(SDL_JoystickGetButton(joystick_, 2) == SDL_PRESSED) return true;
    return false;
}

JoystickInput::JoystickInput(SDL_Joystick* joystick)
    : joystick_(joystick) {}

JoystickInput::~JoystickInput() {
}

InputMgr::~InputMgr() {
    delete_clear(inputs_);
}

void InputMgr::setDevice(Input* input) {
    inputs_.push_back(input);
}

Input::Axis InputMgr::getAxis() const {
    Axis result = NONE;
    std::vector<Input*>::const_iterator ite;
    for (ite = inputs_.begin(); ite != inputs_.end(); ite++) {
	const Input* input = *ite;
	result = input->getAxis();
	if (result != NONE) return result;
    }
    return result;
}

bool InputMgr::getDecision() const {
    if (inputs_.end() != std::find_if(inputs_.begin(), inputs_.end(),
				      std::mem_fun(&Input::getDecision)))
	return true;
    return false;
}

bool InputMgr::getCancel() const {
    if (inputs_.end() != std::find_if(inputs_.begin(), inputs_.end(),
				      std::mem_fun(&Input::getCancel)))
	return true;
    return false;
}

bool InputMgr::getAButton() const {
    if (inputs_.end() != std::find_if(inputs_.begin(), inputs_.end(),
				      std::mem_fun(&Input::getAButton)))
	return true;
    return false;
}

bool InputMgr::getBButton() const {
    if (inputs_.end() != std::find_if(inputs_.begin(), inputs_.end(),
				      std::mem_fun(&Input::getBButton)))
	return true;
    return false;
}

bool InputMgr::getCButton() const {
    if (inputs_.end() != std::find_if(inputs_.begin(), inputs_.end(),
				      std::mem_fun(&Input::getCButton)))
	return true;
    return false;
}

bool InputMgr::waitDecision() const {
    do {
	SDL_PumpEvents();
	if (getCancel()) return false;
    } while (!getDecision());

    do {
	SDL_PumpEvents();
    } while (getDecision());

    return true;
}

bool InputState::getAny() const {
    if (axis_ != NONE ||
	decision_ || cancel_ || aButton_ || bButton_ || cButton_)
	return true;

    return false;
}

InputState InputMgr::waitAnyInput() const {
    InputState state(*this);
    while (!state.getAny()) {
	SDL_PumpEvents();
	state = InputState(*this);
    }

    InputState tmpState(*this);
    while (tmpState.getAny()) {
	SDL_PumpEvents();
	tmpState = InputState(*this);
    }

    return state;
}

InputState::InputState(const Input& input)
    : axis_(input.getAxis()),
      decision_(input.getDecision()),
      cancel_(input.getCancel()),
      aButton_(input.getAButton()),
      bButton_(input.getBButton()),
      cButton_(input.getCButton())
{}

Input::Axis InputState::getAxis() const {
    return axis_;
}

bool InputState::getDecision() const {
    return decision_;
}

bool InputState::getCancel() const {
    return cancel_;
}

bool InputState::getAButton() const {
    return aButton_;
}

bool InputState::getBButton() const {
    return bButton_;
}

bool InputState::getCButton() const {
    return cButton_;
}

