#ifndef __KEYBOARDWM_LOADCONF_H__
#define __KEYBOARDWM_LOADCONF_H__

#include <fstream>
#include <string>
#include <sstream>
#include <stdexcept>

class ConfMapGenerater {
public:
    ConfMapGenerater();

    /// load "key = val" or "key val" style config file.
    /**
     * @param map
     *  STL std::map<Key, Val>
     *  class Key and Val must be able to change from std::string,
     *  generally, Key and Val are std::string.
     */
    //@{
    /**
     * @param str config file's path
     */
    template<class _Map>
    void generate(const std::string& str, _Map& map);
    /**
     * @param is config stream
     */
    template<class _Map>
    void generate(std::istream& is, _Map& map);
    //@}

public:
    void setReportDevice(std::ostream& os) { _report = &os; }

private:
    std::ostream* _report;
};

template<class _Map>
void ConfMapGenerater::generate(const std::string& str, _Map& map) {
    std::ifstream is(str.c_str());
    if (!is) {
		if (_report)
			*_report << str
					 << ": file not found in generateConfMap" << std::endl;
		return;
    }
    else {
		return generate(is, map);
    }
}

template<class _Map>
void ConfMapGenerater::generate(std::istream& is, _Map& map) {
    static const int MAXBUF = 256;
    char ln[MAXBUF];

    while (is.getline(ln, MAXBUF)) {
		std::string line(ln);
		unsigned int s;

		// Ȥν
		if ((s = line.find("#")) != std::string::npos)
			line.erase(s);

		// Ƭν
		if (!line.empty() && line[0] == ' ')
			line.erase(0, line.find_first_not_of(' '));

		std::istringstream ss(line);
		std::string key, val;
		ss >> key >> val;
		if (val == "=") ss >> val;

		while (ss) {
			std::string add;
			ss >> add;
			if (add != "") val += " " + add;
		}

		if (key != "") {
			try {
				map[key] = val;
			}
			catch (std::runtime_error re) {
#ifndef BORLAND
				if (_report) *_report << re.what() << std::endl;
#endif
			}
		}
    }
}

#endif // ! __KEYBOARDWM_LOADCONF_H__

