#include "player.h"
#include "enemy.h"
#include "conf.h"
#include "game.h"
#include "shot.h"
#include "info.h"
#include "win.h"
#include "sprite.h"
#include "art.h"
#include "buzzeffect.h"

Color PlayerType::color() const {
	switch (type_) {
	case RED:
		return Color::RED;
	case ORANGE:
		return Color::ORANGE;
	case YELLOW:
		return Color::YELLOW;
	case GREEN:
		return Color::GREEN;
	case BLUE:
		return Color::BLUE;
	case PURPLE:
		return Color::PURPLE;
	}

	return Color::WHITE;
}

Player::Player (const Point& pnt, Window* graph,
				MSDL::InputBase& inputMgr, CharColor col, PlayerType type,
				int lives)
    : Charactor(pnt, graph, col), pow_(100), inputMgr_(inputMgr),
	  eat_(400), buzz_(0), lives_(lives), blink_(0), hard_(false),
      eblv(5), rankhack(0.0), x_pressed(0), type_(type),
	  pilable_(true), flipping_(false), killing_(false),
	  differBulletAbsorping_(false), differBulletFlipping_(false),
	  sameBulletFlipping_(false), doping_(false)
{
	zako_ = false;

	auraGraphic_.reset(new Window(*Game::instance()->playerGraphic()));
	auraGraphic_->setMask(type.color());
}

Point Player::upperPoint() const {
	return surface_->size() - size();
}

void Player::hitByPlayer (Player*) {
	assert(0);
}

void Player::hitByEnemy (Enemy* bc) {
	nearFromShot(bc);
}

void Player::hitByShot (Shot* bc) {
	nearFromShot(bc);
}

void Player::die() {
	if (lives_ == 0) {
	    std::string wins;
		if (col_ == WHITE) wins = "BLACK";
		else wins = "WHITE";
		surface_->putString(wins + " WINS.", Point(150, 200),
			                opposite()->type().color());
		surface_->flip();
		Game::instance()->end();
	}
	else {
          lives_--;
          {
            /* delete all opponent bullets */
            std::vector<Charactor*> bullets;
            Game::instance()->getShot(col_.reverse(), bullets);
            for (std::vector<Charactor*>::iterator ite = bullets.begin();
                 ite != bullets.end(); ++ite)
            {
              (*ite)->die();
            }
          }
          {
            /* paint around */
            {
              Window tmp(surface_->size(), 16, SDL_SWSURFACE);
              tmp.fillRect(col_.color());
              tmp.setAlpha(SDL_SRCALPHA|SDL_RLEACCEL, 128);
              surface_->source()->blit(&tmp);
            }
            {
              const Point& pnt = center();
              Point size(100, 100);
              surface_->source()->fillRect(Rect(pnt-size, pnt+size),
                                           col_.color());
            }
          }
          
		blink_ = Game::instance()->ticks() + 3000;
		graph_->setBlink(true);
		if (normalSprite_.get() != 0) normalSprite_->setBlink(true);
	}
}

bool Player::deadCheck(Charactor* bc) {
	if (center().length(bc->center()) < 2 && blink_ == 0 &&
		!differBulletAbsorping_ && !differBulletFlipping_
            && (!killing_)) {
		die();
		return true;
	}

	return false;
}


void Player::nearFromShot(Charactor* bc) {
	if (!bc->isZako()) {
		deadCheck(bc);
	}
	else if (color() != bc->color()) {
		if (!deadCheck(bc) && (blink_ == 0) && (!killing_)) {
			if (differBulletAbsorping_) {
				eat(bc);
				bc->die();
			}
			else if (differBulletFlipping_) {
				Point vec(bc->pnt() - pnt_);
				vec.unit();
				bc->setSpd(vec * bc->spd().length() * 1.5);
			}
			else {
				Buzzable* shot = dynamic_cast<Buzzable*>(bc);
				if (shot != 0 && !shot->buzzed()) {
					shot->setBuzzed();
					Game::instance()->addBuzzEffect(
						std::auto_ptr<BuzzEffect>(
							new BuzzEffect(this, bc->center(), bc->color())));
                                        if (eblv <= 4)
                                          increaseRank(2.0);
                                        else
                                          increaseRank(1.0);

                                        if (eblv <= 2)
                                        {
                                          eat_ += 5;
                                          if (eat_ >= 500)
                                            eat_ = 500;
                                        }

					if (buzz_ != 100) {
						buzz_++;
                                                /*
						Info::instance()->noticeBuzz(buzz_, col_);
                                                */
					}
				}
			}
		}
	}
	else {
		if (sameBulletFlipping_) {
			Point vec(bc->pnt() - pnt_);
			vec.unit();
			bc->setSpd(vec * bc->spd().length() * 1.5);
		}
		else {
			eat(bc);
			bc->die();
		}
	}
}

void Player::eat(Charactor* bc) {
	Game::instance()->addBuzzEffect(
		std::auto_ptr<BuzzEffect>(
			new BuzzEffect(this, bc->center(), bc->color())));

        if (doping_) return;

        if ((eblv <= 5) || (eblv >= 10))
          increaseRank(0.2);

	if (eat_ < 500) {
		eat_++;
		if (eat_ % 5 == 0) {
                  /*
			Info::instance()->noticeEat(getEat(), col_);
                  */
                  ;
		}
	}
}

void Player::increaseRank(double c)
{
  /* number of bars | $rank
   * ---------------+------
   *   0            | 0.0
   *   2            | 0.3
   *   4            | 0.5
   *  12            | 1.0
   *
   * linear-interpolate the rest
   */

  if (rankhack < 0.3)
  {
    rankhack += 0.0003 * c;
  }   
  else if (rankhack < 0.5)
  {
    rankhack += 0.0002 * c;
  }
  else
  {
    rankhack += 0.000125 * c;
    if (rankhack > 0.99999)
      rankhack = 1.0;
  }
}

double Player::getRank()
{
  if (doping_)
    return 1.0;
  
  if (rankhack < 0.00001)
    return 0.0;
  else if  (rankhack > 0.99999)
    return 1.0;

  return rankhack;
}

void Player::event() {
	miscEvent();
	keyEvent();
	artEvent();
        /* braindamage for the blink hack */
	Info::instance()->noticeChangeBuzz(buzz_, col_);
        Info::instance()->noticeChangeEat(getEat(), col_);
}

void Player::artEvent() {
	if (art_.get() == 0) return;

	art_->event();

	if (art_->isEnd()) {
		art_.reset(0);
		return;
	}
}

void Player::miscEvent() {
	if (blink_ != 0) {
		if (Game::instance()->ticks() > blink_) {
			blink_ = 0;
			graph_->setBlink(false);
			if (normalSprite_.get() != 0) normalSprite_->setBlink(false);
		}
	}
}

void Player::keyEvent() {
	Game* game = Game::instance();

        if (!killing_)
          spd_ = Point(0, 0);

	SDL_PumpEvents();

	static const double baseSpeed = 150;
	double base = baseSpeed;

	if (inputMgr_.getButton(0)) base *= 0.5;
        if ((differBulletFlipping_) && (sameBulletFlipping_))
        {
          /* green level 3 art */
          base *= 2.0;
        }
        
	double cross = base / SQRT_TWO;

	MSDL::Axis axis = inputMgr_.getAxis();

        if (!killing_)
        {
          if (axis == MSDL::Axis::UP)	spd_.y = -base;
          else if (axis == MSDL::Axis::RIGHT) spd_.x = base;
          else if (axis == MSDL::Axis::DOWN) spd_.y = base;
          else if (axis == MSDL::Axis::LEFT) spd_.x = -base;
          else if (axis == MSDL::Axis::UPRIGHT) {
            spd_.x = cross;
            spd_.y = -cross;
          }
          else if (axis == MSDL::Axis::DOWNRIGHT) {
            spd_.x = cross;
            spd_.y = cross;
          }
          else if (axis == MSDL::Axis::DOWNLEFT) {
            spd_.x = -cross;
            spd_.y = cross;
          }
          else if (axis == MSDL::Axis::UPLEFT) {
            spd_.x = -cross;
            spd_.y = -cross;
          }
        }

	if (inputMgr_.getButton(1))
        {
          if (x_pressed == 0)
          {
            /* the x key doesn't auto-repeat */
            x_pressed = 1;
            if (eat_ >= 500)
            {
              /* spend the eating bar to grow the partner */
              if ((eblv >= 10) || (inputMgr_.getButton(0)))
              {
                /* one more partner */
                Game::instance()->addPartner(col_);
                /* partner level down */
                /* this must be after addPartner() because
                 * the new partner comes with lv5 eb
                 */
                eblv -= 1;
                if (eblv <= 0)
                  eblv = 0;
                Game::instance()->setPartnerLevel(col_, eblv);
              }
              else
              {
                /* partner level up */
                eblv += 1;
                if (eblv >= 10)
                  eblv = 10;
                Game::instance()->setPartnerLevel(col_, eblv);

                {
                  /* ask them new bullets */
                  std::vector<Enemy*> partners;
                  Game::instance()->getPartner(col_, partners);

                  std::for_each(partners.begin(), partners.end(),
                                std::mem_fun(&Enemy::endCommand));
                }
                
                surface_->fillRect(col_.color());
              }
              /* this must be after all the input checks */
              eat_ = 0;
              /*
              Info::instance()->noticeChangeEat(eat_, col_);
              */
            }
            else
            {
              art();
            }
          }
        }
        else
        {
          x_pressed = 0;
        }
        
	if (inputMgr_.getButton(3)) game->end();
}

Player::~Player() {}

void Player::art() {
	// ɬȯưǤʤ
	if (art_.get() != 0) return;

	int level;
	if (buzz_ == 100) {
		level = 3;
		buzz_ = 0;
	}
	else if (buzz_ > 60) {
		level = 2;
		buzz_ -= 60;
	}
	else if (buzz_ > 30) {
		level = 1;
		buzz_ -= 30;
	}
	else {
		return;
	}
        /*
	Info::instance()->noticeChangeBuzz(buzz_, col_);
        */

	art_.reset(Art::create(this, level, surface_).release());
}

Player* Player::opposite() const {
	return Game::instance()->player(col_.reverse());
}

void Player::setEat(int eat) {
	eat_ = eat * 5;
	if (eat_ > 500) eat_ = 500;
	if (eat_ < 0) eat_ = 0;
        /*
	Info::instance()->noticeChangeEat(getEat(), col_);
        */
}

void Player::moveValue() {
	if (isHit(opposite())) {
		if (!opposite()->getPilable()) {
			return;
		}
		else if (opposite()->getFlipping()) {
			pnt_ += opposite()->spd() * 2 * Game::instance()->spf();
			calcCenter();
		}
		else if (opposite()->getKilling()) {
			if (center().length(opposite()->center()) < 24 && blink_ == 0) {
				die();
			}
		}
	}

	Charactor::moveValue();
}

void Player::setAura() {
	normalSprite_.reset(graph_.release());
	graph_.reset(new Sprite(auraGraphic_.get()));
}

void Player::unsetAura() {
	graph_.reset(normalSprite_.release());
}
