#ifndef BOHA_PLAYER
#define BOHA_PLAYER

#include "config.h"
#include "char.h"
#include "util.h"
#include "mysdl_padinput.h"

class CpuInput;

class PlayerType {
public:
	typedef enum {
		RED =0, ORANGE, YELLOW, GREEN, BLUE, PURPLE
	} PlayerType_;

public:
	PlayerType() {}
	PlayerType(PlayerType_ type) : type_(type) {}
	operator PlayerType_() { return type_; }

	Color color() const;

private:
	PlayerType_ type_;

};

class Art;

class Player : public Charactor {
public:
    Player(const Point& pnt, Window* graph,
		   MSDL::InputBase& inputMgr, CharColor col,
		   PlayerType type, int lives =0);
    virtual ~Player();

    void event();

    virtual void hit (Charactor* bc) { bc->hitByPlayer(this); }
    virtual void hitByPlayer (Player* bc);
    virtual void hitByEnemy (Enemy* bc);
    virtual void hitByShot (Shot* bc);

    Point upperPoint() const;

    double getRank();

	int buzz() const { return buzz_; }

	int getEat() const { return eat_ / 5; }
	void setEat(int eat);

	Player* opposite() const;

	PlayerType type() const { return type_; }

	virtual void die();

	void eat(Charactor* bc);
	
	int lives() const { return lives_; }

        void increaseRank(double c);

        int getXPressed() const { return x_pressed; }
        int getEblv() const { return eblv; }

private:
	void miscEvent();
	void keyEvent();
	void artEvent();
	void art();

private:
	void nearFromShot(Charactor* bc);
	bool deadCheck(Charactor* bc);

private:
	virtual void moveValue();

private:
    double prevShotTurn_;

    double pow_;

    MSDL::InputBase& inputMgr_;

	int eat_;
	int buzz_;
	int lives_;
	Uint32 blink_;

	bool hard_;

        int eblv;
        double rankhack;
        int x_pressed;

private:
	PlayerType type_;

private:
	std::auto_ptr<Art> art_;

private:
	std::auto_ptr<Window> auraGraphic_;
	std::auto_ptr<Sprite> normalSprite_;

public:
	void setAura();
	void unsetAura();

	/// ʲɬΤʤ󤫵뤤
	/**
	 * ̿̾§㤯
	 */
	//@{

public:
	void setPilable(bool pilable) { pilable_ = pilable; }
	bool getPilable() const { return pilable_; }
	void setFlipping(bool flipping) { flipping_ = flipping; }
	bool getFlipping() const { return flipping_; }
	void setKilling(bool killing) { killing_ = killing; }
	bool getKilling() const { return killing_; }
	void setDifferBulletAbsorping(bool dba) { differBulletAbsorping_ = dba; }
	bool getDifferBulletAbsorping() const { return differBulletAbsorping_; }
	void setDifferBulletFlipping(bool dbf) { differBulletFlipping_ = dbf; }
	bool getDifferBulletFlipping() const { return differBulletFlipping_; }
	void setSameBulletFlipping(bool sbf) { sameBulletFlipping_ = sbf; }
	bool getSameBulletFlipping() const { return sameBulletFlipping_; }
	void setDoping(bool doping) { doping_ = doping; }
	bool getDoping() const { return doping_; }

private:
	bool pilable_;
	bool flipping_;
	bool killing_;
	bool differBulletAbsorping_;
	bool differBulletFlipping_;
	bool sameBulletFlipping_;
	bool doping_;
	//@}

};

#endif // ! BOHA_PLAYER
