#ifndef BOHA_UTIL
#define BOHA_UTIL

#include "config.h"

#include <SDL.h>

#include <string>
#include <stdexcept>
#include <cmath>
#include <iostream>

/// xyνͤ͡ǤդxФy֤륯饹
template <class X_ = double, class Y_ = double>
class LinearFunc {
public:
    LinearFunc(const X_& firstX, const X_& lastX,
			   const Y_& firstY, const Y_& lastY)
		: firstX_(firstX), lastX_(lastX),
		  firstY_(firstY), lastY_(lastY),
		  gradient_((lastY-firstY)/(lastX-firstX)) {}

    Y_ getValue(const X_& x) {
		return firstY_ + gradient_ * (x-firstX_);
    }

    bool isLast(const X_& x) {
		return x >= lastX_;
    }
    void setLast(Y_& y) {
		y = lastY_;
    }

protected:
    X_ firstX_, lastX_;
    Y_ firstY_, lastY_;
    Y_ gradient_;
};

/// undef դѿ
template <class C_>
class Validatable {
public:
    Validatable() : isValidate_(false) {}

    bool isValidate() const { return isValidate_; }

    void enValidate() { isValidate_ = true; }
    void disValidate() { isValidate_ = false; }

    operator C_& () { return val_; }

    C_& operator = (const C_& rhs) {
		isValidate_ = true;
		val_ = rhs;
		return *this;
    }

protected:
    C_ val_;

    bool isValidate_;
};

template <class C_>
std::istream& operator >> (std::istream& is, Validatable<C_>& rhs) {
    C_ x;
    is >> x;
    rhs = x;
    return is;
}

/**
 * @param
 * Cont_ STLѥʥƥʤǤĥݥ󥿤ǼƤ
 */
template<class Cont_>
void delete_clear(Cont_& cont) {
    typename Cont_::iterator ite;
    for (ite = cont.begin(); ite != cont.end(); ite++) {
		delete *ite;
    }
    cont.clear();
}

const double PI = 3.1415926;
const double SQRT_TWO = std::sqrt(2.0);

int rnd(int x);
double rnd(double x);

double dtor(double x);
double rtod(double x);

void ASSERT(bool t, const char* str);
void ASSERT(bool t, const std::string& str);

void ignoreBrankAndComment(std::string& str, std::string delim = "#");

class PolarCoord;

class Point {
public:
    Point ();
    Point (const Point& rhs);
    Point (const int x1, const int y1);
    Point (const double x1, const double y1);
    explicit Point (const PolarCoord& rhs);

    const Point operator+ (const Point& rhs) const {
		Point p(x+rhs.x, y+rhs.y);
		return p;
    }
    const Point operator- (const Point& rhs) const {
		Point p(x-rhs.x, y-rhs.y);
		return p;
    }
    const Point operator* (const double rhs) const {
		Point p(x*rhs, y*rhs);
		return p;
    }
    const Point operator/ (const double rhs) const {
		Point p(x/rhs, y/rhs);
		return p;
    }
    const Point& operator+= (const Point& rhs) {
		x += rhs.x;
		y += rhs.y;
		return *this;
    }
    const Point& operator-= (const Point& rhs) {
		x -= rhs.x;
		y -= rhs.y;
		return *this;
    }
    const Point& operator*= (const double rhs) {
		x *= rhs;
		y *= rhs;
		return *this;
    }
    const Point& operator/= (const double rhs) {
		x /= rhs;
		y /= rhs;
		return *this;
    }
    const Point operator-() {
		Point p(-x, -y);
		return p;
    }

    double length() const;
    double length(const Point& p) const;
    const Point& unit();

    double angle() const;
    double angle(const Point& p) const;

    double innerProduct() const {
		return x*x+y*y;
    }

    int xi() const { return (int)(x); }
    int yi() const { return (int)(y); }
    double xd() const { return x; }
    double yd() const { return y; }

    double x, y;
};

class PolarCoord {
public:
    PolarCoord();
    PolarCoord(const PolarCoord& rhs);
    PolarCoord(double l0, double r0);
    explicit PolarCoord(const Point& pnt);

    double l, r;
};

class Rect {
public:
    Rect ();
    Rect (const Rect & rhs);
    // if 'to' is true, w = xe 
    Rect (const int xs, const int ys, const int xe, const int ye,
		  const bool to = false);
    Rect (const Point ps, const Point pe, const bool to = false);

    SDL_Rect* val() { return &val_; }
    const SDL_Rect* val() const { return &val_; }

private:
    SDL_Rect val_;
};

class Color {
public:
    Color(int r, int g, int b) {
		col_.r = r;
		col_.g = g;
		col_.b = b;
	}

	int r() const { return col_.r; }
	int g() const { return col_.g; }
	int b() const { return col_.b; }

	SDL_Color col() const { return col_; }

	Uint32 map(SDL_PixelFormat* fmt) {
		return SDL_MapRGB(fmt, col_.r, col_.g, col_.b);
	}

public:
	static const Color RED, ORANGE, YELLOW, GREEN, BLUE, PURPLE;
	static const Color WHITE, BLACK, GRAY;

protected:
	SDL_Color col_;
};

inline std::ostream& operator<< (std::ostream& os, const Point& p) {
	os << '(' << p.xd() << ',' << p.yd() << ')';
	return os;
}

inline std::ostream& operator<< (std::ostream& os, const Color& c) {
	os << '(' << c.r() << ',' << c.g() << ',' << c.b() << ')';
	return os;
}

#endif // BOHA_UTIL
