#ifndef BOHA_WIN
#define BOHA_WIN

#include "config.h"
#include "util.h"
#include "singleton.h"
#include "config.h"

#include <SDL.h>

#ifdef HAVE_SDL_SDL_TTF_H
#include <SDL_ttf.h>
#endif

#include <cassert>

#include <string>
#include <iostream>
#include <utility>
#include <memory>

// ɥ饹
class Window {
public:
	explicit Window(SDL_Surface* surf) : surf_(surf) {}
    explicit Window(const std::string& str);
	Window(const Window& rhs);
	Window(const Point& size, int bpp, Uint32 flag,
		   Uint32 Rmask =0, Uint32 Gmask =0, Uint32 Bmask =0, Uint32 Amask =0);
    Window(int x, int y, int bpp, Uint32 flag,
		   Uint32 Rmask =0, Uint32 Gmask =0, Uint32 Bmask =0, Uint32 Amask =0);

    virtual ~Window();

    void fillRect(const Rect& rect, int color) {
		int ret = SDL_FillRect(surf_, const_cast<SDL_Rect*>(rect.val()),
							   color);
		ASSERT(ret == 0, "Window::fillRect");
    }
	void fillRect(const Rect& rect, Color col) {
		fillRect(rect, mapRGB(col));
	}
	void fillRect(int color) {
		int ret = SDL_FillRect(surf_, 0, color);
		ASSERT(ret == 0, "Window::fillRect");
	}
	void fillRect(Color col) {
		fillRect(mapRGB(col));
	}

    void blit(Window* src) {
		int ret = SDL_BlitSurface(src->surf_, 0, surf_, 0);
		ASSERT(ret == 0, "Window::blit");
    }
    void blit(Window* src, const Rect& srcrect, const Rect& dstrect) {
		int ret =
			SDL_BlitSurface(src->surf_, const_cast<SDL_Rect*>(srcrect.val()),
							surf_, const_cast<SDL_Rect*>(dstrect.val()));
		ASSERT(ret == 0, "Window::blit");
    }
	void blit(Window* src, const Point& pnt) {
		SDL_Rect dst;
		dst.x = pnt.xi();
		dst.y = pnt.yi();
		int ret =
			SDL_BlitSurface(src->surf_, 0, surf_, &dst);
		ASSERT(ret == 0, "Window::blit");
	}

	void stretch(Window* src) {
		SDL_SoftStretch(src->surf_, 0, surf_, 0);
	}

    void printInformation(std::ostream& os) const;

    virtual const Point& size() const { return size_; }

    void save(const std::string& filename) {
		SDL_SaveBMP(surf_, filename.c_str());
    }

	void setAlpha(Uint32 flag, Uint8 alpha) {
		int ret = SDL_SetAlpha(surf_, flag, alpha);
		ASSERT(ret == 0, "Window::setAlpha");
	}

	void convertDisplayFormat();

    void setColorKey(Uint32 flag, Uint32 key) {
		int ret = SDL_SetColorKey(surf_, flag, key);
		ASSERT(ret == 0, "WindowsetColorKey");
    }
	void setColorKey(Uint32 flag, Color col) {
		setColorKey(flag, mapRGB(col));
	}

	void setMask(Color col);

	double halfDiagonal() const { return halfDiagonal_; }
	const Point& halfSize() const { return halfSize_; }

	Uint32 getPixel(int x, int y);
	void putPixel(int x, int y, Uint32 pixel);

    Uint32 mapRGB(Color col) const {
		return col.map(surf_->format);
    }
	Color getRGB(Uint32 pixel) const {
		Uint8 r, g, b;
		SDL_GetRGB(pixel, surf_->format, &r, &g, &b);
		return Color(r, g, b);
	}

	void lock() {
		SDL_LockSurface(surf_);
	}
	void unlock() {
		SDL_UnlockSurface(surf_);
	}

protected:
	void calcCache();

protected:
	Window() : surf_(0) {}

    SDL_Surface* surf_;
    Point size_;

	double halfDiagonal_;
	Point halfSize_;

};

// ᥤ󥦥ɥ
class MainWindow : public Window, public Singleton<MainWindow> {
public:
    explicit MainWindow(const std::string& str);
    MainWindow(const std::string& str, int x, int y, Uint32 addFlag =0);

private:
	void init(const std::string& str);

public:
	virtual ~MainWindow();

    void loadBMP(const std::string& str);
    void loadBMP(const std::string& str, const Point& pnt);

    void putString(const std::string& str, const Point& pnt,
				   const Color& col);

    virtual const Point& size() const { return first_->size(); }

    SDL_PixelFormat* format() { return surf_->format; }

    int flip();

    void clear();
    void clearRect(const Rect& rect);

	void reset();

	Window* source() { return first_.get(); }

	void display();

protected:
    static const Uint32 flag_;

    std::auto_ptr<Window> first_;
	std::auto_ptr<Window> orig_;
	SDL_Surface* screen_;

#ifdef HAVE_SDL_SDL_TTF_H
	TTF_Font* font_;
#else
    std::auto_ptr<Window> strings_;
    std::string sample_;
    static const double dotPerChar_;
    static const int charWidth_;
    static const int charHeight_;
#endif
};

#endif // BOHA_WIN
